package java.awt.image;

import java.util.Arrays;
import java.awt.color.ColorSpace;
import sun.awt.image.BufImgSurfaceData;
import java.math.BigInteger;

public class IndexColorModel extends ColorModel
{
    private int[] rgb;
    private int map_size;
    private int pixel_mask;
    private int transparent_index;
    private boolean allgrayopaque;
    private BigInteger validBits;
    private BufImgSurfaceData.ICMColorData colorData;
    private static int[] opaqueBits;
    private static int[] alphaBits;
    private static final int CACHESIZE = 40;
    private int[] lookupcache;
    
    private static native void initIDs();
    
    public IndexColorModel(int n, int n2, byte[] array, byte[] array2, byte[] array3) {
        super(n, IndexColorModel.opaqueBits, ColorSpace.getInstance(1000), false, false, 1, ColorModel.getDefaultTransferType(n));
        this.transparent_index = -1;
        this.colorData = null;
        this.lookupcache = new int[40];
        if (n < 1 || n > 16) {
            throw new IllegalArgumentException("Number of bits must be between 1 and 16.");
        }
        this.setRGBs(n2, array, array2, array3, (byte[])null);
        this.calculatePixelMask();
        return;
    }
    
    public IndexColorModel(int n, int n2, byte[] array, byte[] array2, byte[] array3, int transparentPixel) {
        super(n, IndexColorModel.opaqueBits, ColorSpace.getInstance(1000), false, false, 1, ColorModel.getDefaultTransferType(n));
        this.transparent_index = -1;
        this.colorData = null;
        this.lookupcache = new int[40];
        if (n < 1 || n > 16) {
            throw new IllegalArgumentException("Number of bits must be between 1 and 16.");
        }
        this.setRGBs(n2, array, array2, array3, (byte[])null);
        this.setTransparentPixel(transparentPixel);
        this.calculatePixelMask();
        return;
    }
    
    public IndexColorModel(int n, int n2, byte[] array, byte[] array2, byte[] array3, byte[] array4) {
        super(n, IndexColorModel.alphaBits, ColorSpace.getInstance(1000), true, false, 3, ColorModel.getDefaultTransferType(n));
        this.transparent_index = -1;
        this.colorData = null;
        this.lookupcache = new int[40];
        if (n < 1 || n > 16) {
            throw new IllegalArgumentException("Number of bits must be between 1 and 16.");
        }
        this.setRGBs(n2, array, array2, array3, array4);
        this.calculatePixelMask();
        return;
    }
    
    public IndexColorModel(int n, int n2, byte[] array, int n3, boolean b) {
        this(n, n2, array, n3, b, -1);
        if (n < 1 || n > 16) {
            throw new IllegalArgumentException("Number of bits must be between 1 and 16.");
        }
        return;
    }
    
    public IndexColorModel(int n, int n2, byte[] array, int n3, boolean b, int transparentPixel) {
        super(n, IndexColorModel.opaqueBits, ColorSpace.getInstance(1000), false, false, 1, ColorModel.getDefaultTransferType(n));
        this.transparent_index = -1;
        this.colorData = null;
        this.lookupcache = new int[40];
        if (n < 1 || n > 16) {
            throw new IllegalArgumentException("Number of bits must be between 1 and 16.");
        }
        if (n2 < 1) {
            throw new IllegalArgumentException("Map size (" + n2 + ") must be >= 1");
        }
        this.map_size = n2;
        this.rgb = new int[this.calcRealMapSize(n, n2)];
        int n4 = 255;
        boolean allgrayopaque = true;
        int transparency = 1;
        int n5;
        int n6;
        int n7;
        for (int i = 0; i < n2; this.rgb[i] = (n4 << 24 | n5 << 16 | n6 << 8 | n7), ++i) {
            n5 = (array[n3++] & 0xFF);
            n6 = (array[n3++] & 0xFF);
            n7 = (array[n3++] & 0xFF);
            allgrayopaque = (allgrayopaque && n5 == n6 && n6 == n7);
            if (b) {
                n4 = (array[n3++] & 0xFF);
                if (n4 != 255) {
                    if (n4 == 0) {
                        if (transparency == 1) {
                            transparency = 2;
                        }
                        if (this.transparent_index < 0) {
                            this.transparent_index = i;
                        }
                    }
                    else {
                        transparency = 3;
                    }
                    allgrayopaque = false;
                }
            }
        }
        this.allgrayopaque = allgrayopaque;
        this.setTransparency(transparency);
        this.setTransparentPixel(transparentPixel);
        this.calculatePixelMask();
        return;
    }
    
    public IndexColorModel(int n, int i, int[] array, int n2, boolean b, int transparentPixel, int n3) {
        super(n, IndexColorModel.opaqueBits, ColorSpace.getInstance(1000), false, false, 1, n3);
        this.transparent_index = -1;
        this.colorData = null;
        this.lookupcache = new int[40];
        if (n < 1 || n > 16) {
            throw new IllegalArgumentException("Number of bits must be between 1 and 16.");
        }
        if (i < 1) {
            throw new IllegalArgumentException("Map size (" + i + ") must be >= 1");
        }
        if (n3 != 0 && n3 != 1) {
            throw new IllegalArgumentException("transferType must be eitherDataBuffer.TYPE_BYTE or DataBuffer.TYPE_USHORT");
        }
        this.setRGBs(i, array, n2, b);
        this.setTransparentPixel(transparentPixel);
        this.calculatePixelMask();
        return;
    }
    
    public IndexColorModel(int n, int i, int[] array, int n2, int n3, BigInteger validBits) {
        super(n, IndexColorModel.alphaBits, ColorSpace.getInstance(1000), true, false, 3, n3);
        this.transparent_index = -1;
        this.colorData = null;
        this.lookupcache = new int[40];
        if (n < 1 || n > 16) {
            throw new IllegalArgumentException("Number of bits must be between 1 and 16.");
        }
        if (i < 1) {
            throw new IllegalArgumentException("Map size (" + i + ") must be >= 1");
        }
        if (n3 != 0 && n3 != 1) {
            throw new IllegalArgumentException("transferType must be eitherDataBuffer.TYPE_BYTE or DataBuffer.TYPE_USHORT");
        }
        if (validBits != null) {
            int j = 0;
            while (j < i) {
                if (!validBits.testBit(j)) {
                    this.validBits = validBits;
                    break;
                }
                ++j;
            }
        }
        this.setRGBs(i, array, n2, true);
        this.calculatePixelMask();
        return;
    }
    
    private void setRGBs(int n, byte[] array, byte[] array2, byte[] array3, byte[] array4) {
        if (n < 1) {
            throw new IllegalArgumentException("Map size (" + n + ") must be >= 1");
        }
        this.map_size = n;
        this.rgb = new int[this.calcRealMapSize(this.pixel_bits, n)];
        int n2 = 255;
        int transparency = 1;
        boolean allgrayopaque = true;
        int n3;
        int n4;
        int n5;
        for (int i = 0; i < n; this.rgb[i] = (n2 << 24 | n3 << 16 | n4 << 8 | n5), ++i) {
            n3 = (array[i] & 0xFF);
            n4 = (array2[i] & 0xFF);
            n5 = (array3[i] & 0xFF);
            allgrayopaque = (allgrayopaque && n3 == n4 && n4 == n5);
            if (array4 != null) {
                n2 = (array4[i] & 0xFF);
                if (n2 != 255) {
                    if (n2 == 0) {
                        if (transparency == 1) {
                            transparency = 2;
                        }
                        if (this.transparent_index < 0) {
                            this.transparent_index = i;
                        }
                    }
                    else {
                        transparency = 3;
                    }
                    allgrayopaque = false;
                }
            }
        }
        this.allgrayopaque = allgrayopaque;
        this.setTransparency(transparency);
        return;
    }
    
    private void setRGBs(int map_size, int[] array, int n, boolean b) {
        this.map_size = map_size;
        this.rgb = new int[this.calcRealMapSize(this.pixel_bits, map_size)];
        int transparency = 1;
        boolean allgrayopaque = true;
        BigInteger validBits = this.validBits;
        for (int i = 0; i < map_size; ++i, ++n) {
            if (validBits == null || validBits.testBit(i)) {
                int n2 = array[n];
                int n3 = n2 >> 16 & 0xFF;
                int n4 = n2 >> 8 & 0xFF;
                int n5 = n2 & 0xFF;
                allgrayopaque = (allgrayopaque && n3 == n4 && n4 == n5);
                if (b) {
                    int n6 = n2 >>> 24;
                    if (n6 != 255) {
                        if (n6 == 0) {
                            if (transparency == 1) {
                                transparency = 2;
                            }
                            if (this.transparent_index < 0) {
                                this.transparent_index = i;
                            }
                        }
                        else {
                            transparency = 3;
                        }
                        allgrayopaque = false;
                    }
                }
                else {
                    n2 |= 0xFF000000;
                }
                this.rgb[i] = n2;
            }
        }
        this.allgrayopaque = allgrayopaque;
        this.setTransparency(transparency);
    }
    
    private int calcRealMapSize(int n, int b) {
        return Math.max(Math.max(1 << n, b), 256);
    }
    
    private BigInteger getAllValid() {
        int n = (this.map_size + 7) / 8;
        byte[] array = new byte[n];
        Arrays.fill(array, (byte)(-1));
        array[0] = (byte)(255 >>> n * 8 - this.map_size);
        return new BigInteger(1, array);
    }
    
    @Override
    public int getTransparency() {
        return this.transparency;
    }
    
    @Override
    public int[] getComponentSize() {
        if (this.nBits == null) {
            if (this.supportsAlpha) {
                (this.nBits = new int[4])[3] = 8;
            }
            else {
                this.nBits = new int[3];
            }
            int[] nBits = this.nBits;
            int n = 0;
            int[] nBits2 = this.nBits;
            int n2 = 1;
            int[] nBits3 = this.nBits;
            int n3 = 2;
            int n4 = 8;
            nBits3[n3] = n4;
            nBits[n] = (nBits2[n2] = n4);
        }
        return (int[])this.nBits.clone();
    }
    
    public final int getMapSize() {
        return this.map_size;
    }
    
    public final int getTransparentPixel() {
        return this.transparent_index;
    }
    
    public final void getReds(byte[] array) {
        for (int i = 0; i < this.map_size; ++i) {
            array[i] = (byte)(this.rgb[i] >> 16);
        }
    }
    
    public final void getGreens(byte[] array) {
        for (int i = 0; i < this.map_size; ++i) {
            array[i] = (byte)(this.rgb[i] >> 8);
        }
    }
    
    public final void getBlues(byte[] array) {
        for (int i = 0; i < this.map_size; ++i) {
            array[i] = (byte)this.rgb[i];
        }
    }
    
    public final void getAlphas(byte[] array) {
        for (int i = 0; i < this.map_size; ++i) {
            array[i] = (byte)(this.rgb[i] >> 24);
        }
    }
    
    public final void getRGBs(int[] array) {
        System.arraycopy(this.rgb, 0, array, 0, this.map_size);
    }
    
    private void setTransparentPixel(int transparent_index) {
        if (transparent_index >= 0 && transparent_index < this.map_size) {
            int[] rgb = this.rgb;
            rgb[transparent_index] &= 0xFFFFFF;
            this.transparent_index = transparent_index;
            this.allgrayopaque = false;
            if (this.transparency == 1) {
                this.setTransparency(2);
            }
        }
    }
    
    private void setTransparency(int transparency) {
        if (this.transparency != transparency) {
            if ((this.transparency = transparency) == 1) {
                this.supportsAlpha = false;
                this.numComponents = 3;
                this.nBits = IndexColorModel.opaqueBits;
            }
            else {
                this.supportsAlpha = true;
                this.numComponents = 4;
                this.nBits = IndexColorModel.alphaBits;
            }
        }
    }
    
    private final void calculatePixelMask() {
        int pixel_bits = this.pixel_bits;
        if (pixel_bits == 3) {
            pixel_bits = 4;
        }
        else if (pixel_bits > 4 && pixel_bits < 8) {
            pixel_bits = 8;
        }
        this.pixel_mask = (1 << pixel_bits) - 1;
    }
    
    @Override
    public final int getRed(int n) {
        return this.rgb[n & this.pixel_mask] >> 16 & 0xFF;
    }
    
    @Override
    public final int getGreen(int n) {
        return this.rgb[n & this.pixel_mask] >> 8 & 0xFF;
    }
    
    @Override
    public final int getBlue(int n) {
        return this.rgb[n & this.pixel_mask] & 0xFF;
    }
    
    @Override
    public final int getAlpha(int n) {
        return this.rgb[n & this.pixel_mask] >> 24 & 0xFF;
    }
    
    @Override
    public final int getRGB(int n) {
        return this.rgb[n & this.pixel_mask];
    }
    
    @Override
    public synchronized Object getDataElements(int n, Object o) {
        int n2 = n >> 16 & 0xFF;
        int n3 = n >> 8 & 0xFF;
        int n4 = n & 0xFF;
        int n5 = n >>> 24;
        int transparent_index = 0;
        int n6 = 38;
        while (n6 >= 0 && (transparent_index = this.lookupcache[n6]) != 0) {
            if (n == this.lookupcache[n6 + 1]) {
                return this.installpixel(o, ~transparent_index);
            }
            n6 -= 2;
        }
        if (this.allgrayopaque) {
            int n26 = 256;
            int n27 = (n2 * 77 + n3 * 150 + n4 * 29 + 128) / 256;
            for (int l = 0; l < this.map_size; ++l) {
                if (this.rgb[l] != 0) {
                    int n28 = (this.rgb[l] & 0xFF) - n27;
                    if (n28 < 0) {
                        n28 = -n28;
                    }
                    if (n28 < n26) {
                        transparent_index = l;
                        if (n28 == 0) {
                            break;
                        }
                        n26 = n28;
                    }
                }
            }
        }
        else if (this.transparency == 1) {
            int n17 = Integer.MAX_VALUE;
            int[] rgb2 = this.rgb;
            int j = 0;
            while (j < this.map_size) {
                int n18 = rgb2[j];
                if (n18 == n && n18 != 0) {
                    transparent_index = j;
                    n17 = 0;
                    break;
                }
                ++j;
            }
            if (n17 != 0) {
                for (int k = 0; k < this.map_size; ++k) {
                    int n19 = rgb2[k];
                    if (n19 != 0) {
                        int n20 = (n19 >> 16 & 0xFF) - n2;
                        int n21 = n20 * n20;
                        if (n21 < n17) {
                            int n22 = (n19 >> 8 & 0xFF) - n3;
                            int n23 = n21 + n22 * n22;
                            if (n23 < n17) {
                                int n24 = (n19 & 0xFF) - n4;
                                int n25 = n23 + n24 * n24;
                                if (n25 < n17) {
                                    transparent_index = k;
                                    n17 = n25;
                                }
                            }
                        }
                    }
                }
            }
        }
        else if (n5 == 0 && this.transparent_index >= 0) {
            transparent_index = this.transparent_index;
        }
        else {
            int n7 = Integer.MAX_VALUE;
            int[] rgb = this.rgb;
            for (int i = 0; i < this.map_size; ++i) {
                int n8 = rgb[i];
                if (n8 == n) {
                    if (this.validBits == null || this.validBits.testBit(i)) {
                        transparent_index = i;
                        break;
                    }
                }
                else {
                    int n9 = (n8 >> 16 & 0xFF) - n2;
                    int n10 = n9 * n9;
                    if (n10 < n7) {
                        int n11 = (n8 >> 8 & 0xFF) - n3;
                        int n12 = n10 + n11 * n11;
                        if (n12 < n7) {
                            int n13 = (n8 & 0xFF) - n4;
                            int n14 = n12 + n13 * n13;
                            if (n14 < n7) {
                                int n15 = (n8 >>> 24) - n5;
                                int n16 = n14 + n15 * n15;
                                if (n16 < n7 && (this.validBits == null || this.validBits.testBit(i))) {
                                    transparent_index = i;
                                    n7 = n16;
                                }
                            }
                        }
                    }
                }
            }
        }
        System.arraycopy(this.lookupcache, 2, this.lookupcache, 0, 38);
        this.lookupcache[39] = n;
        this.lookupcache[38] = ~transparent_index;
        return this.installpixel(o, transparent_index);
    }
    
    private Object installpixel(Object o, int n) {
        switch (this.transferType) {
            case 3: {
                int[] array;
                if (o == null) {
                    array = (o = new int[] { 0 });
                }
                else {
                    array = (int[])(int[])o;
                }
                array[0] = n;
                break;
            }
            case 0: {
                byte[] array2;
                if (o == null) {
                    array2 = (o = new byte[] { 0 });
                }
                else {
                    array2 = (byte[])(byte[])o;
                }
                array2[0] = (byte)n;
                break;
            }
            case 1: {
                short[] array3;
                if (o == null) {
                    array3 = (o = new short[] { 0 });
                }
                else {
                    array3 = (short[])(short[])o;
                }
                array3[0] = (short)n;
                break;
            }
            default: {
                throw new UnsupportedOperationException("This method has not been implemented for transferType " + this.transferType);
            }
        }
        return o;
    }
    
    @Override
    public int[] getComponents(int n, int[] array, int n2) {
        if (array == null) {
            array = new int[n2 + this.numComponents];
        }
        array[n2 + 0] = this.getRed(n);
        array[n2 + 1] = this.getGreen(n);
        array[n2 + 2] = this.getBlue(n);
        if (this.supportsAlpha && array.length - n2 > 3) {
            array[n2 + 3] = this.getAlpha(n);
            return array;
        }
        return array;
    }
    
    @Override
    public int[] getComponents(Object o, int[] array, int n) {
        int n2;
        switch (this.transferType) {
            case 0: {
                n2 = (((byte[])(byte[])o)[0] & 0xFF);
                break;
            }
            case 1: {
                n2 = (((short[])(short[])o)[0] & 0xFFFF);
                break;
            }
            case 3: {
                n2 = ((int[])(int[])o)[0];
                break;
            }
            default: {
                throw new UnsupportedOperationException("This method has not been implemented for transferType " + this.transferType);
            }
        }
        return this.getComponents(n2, array, n);
    }
    
    @Override
    public int getDataElement(int[] array, int n) {
        int n2 = array[n + 0] << 16 | array[n + 1] << 8 | array[n + 2];
        int n3;
        if (this.supportsAlpha) {
            n3 = (n2 | array[n + 3] << 24);
        }
        else {
            n3 = (n2 | 0xFF000000);
        }
        Object dataElements = this.getDataElements(n3, null);
        int n4;
        switch (this.transferType) {
            case 0: {
                n4 = (((byte[])(byte[])dataElements)[0] & 0xFF);
                break;
            }
            case 1: {
                n4 = ((short[])(short[])dataElements)[0];
                break;
            }
            case 3: {
                n4 = ((int[])(int[])dataElements)[0];
                break;
            }
            default: {
                throw new UnsupportedOperationException("This method has not been implemented for transferType " + this.transferType);
            }
        }
        return n4;
    }
    
    @Override
    public Object getDataElements(int[] array, int n, Object o) {
        int n2 = array[n + 0] << 16 | array[n + 1] << 8 | array[n + 2];
        int n3;
        if (this.supportsAlpha) {
            n3 = (n2 | array[n + 3] << 24);
        }
        else {
            n3 = (n2 & 0xFF000000);
        }
        return this.getDataElements(n3, o);
    }
    
    @Override
    public WritableRaster createCompatibleWritableRaster(int n, int n2) {
        WritableRaster writableRaster;
        if (this.pixel_bits == 1 || this.pixel_bits == 2 || this.pixel_bits == 4) {
            writableRaster = Raster.createPackedRaster(0, n, n2, 1, this.pixel_bits, null);
        }
        else if (this.pixel_bits <= 8) {
            writableRaster = Raster.createInterleavedRaster(0, n, n2, 1, null);
        }
        else if (this.pixel_bits <= 16) {
            writableRaster = Raster.createInterleavedRaster(1, n, n2, 1, null);
        }
        else {
            throw new UnsupportedOperationException("This method is not supported  for pixel bits > 16.");
        }
        return writableRaster;
    }
    
    @Override
    public boolean isCompatibleRaster(Raster raster) {
        int sampleSize = raster.getSampleModel().getSampleSize(0);
        return raster.getTransferType() == this.transferType && raster.getNumBands() == 1 && 1 << sampleSize >= this.map_size;
    }
    
    @Override
    public SampleModel createCompatibleSampleModel(int n, int n2) {
        int[] array = { 0 };
        if (this.pixel_bits == 1 || this.pixel_bits == 2 || this.pixel_bits == 4) {
            return new MultiPixelPackedSampleModel(this.transferType, n, n2, this.pixel_bits);
        }
        return new ComponentSampleModel(this.transferType, n, n2, 1, n, array);
    }
    
    @Override
    public boolean isCompatibleSampleModel(SampleModel sampleModel) {
        return (sampleModel instanceof ComponentSampleModel || sampleModel instanceof MultiPixelPackedSampleModel) && sampleModel.getTransferType() == this.transferType && sampleModel.getNumBands() == 1;
    }
    
    public BufferedImage convertToIntDiscrete(Raster raster, boolean b) {
        if (!this.isCompatibleRaster(raster)) {
            throw new IllegalArgumentException("This raster is not compatiblewith this IndexColorModel.");
        }
        ColorModel rgBdefault;
        if (b || this.transparency == 3) {
            rgBdefault = ColorModel.getRGBdefault();
        }
        else if (this.transparency == 2) {
            rgBdefault = new DirectColorModel(25, 16711680, 65280, 255, 16777216);
        }
        else {
            rgBdefault = new DirectColorModel(24, 16711680, 65280, 255);
        }
        int width = raster.getWidth();
        int height = raster.getHeight();
        WritableRaster compatibleWritableRaster = rgBdefault.createCompatibleWritableRaster(width, height);
        Object dataElements = null;
        int minX = raster.getMinX();
        for (int minY = raster.getMinY(), i = 0; i < height; ++i, ++minY) {
            dataElements = raster.getDataElements(minX, minY, width, 1, dataElements);
            int[] intArray;
            if ((dataElements instanceof int[])) {
                intArray = (int[])(int[])dataElements;
            }
            else {
                intArray = DataBuffer.toIntArray(dataElements);
            }
            for (int j = 0; j < width; ++j) {
                intArray[j] = this.rgb[intArray[j] & this.pixel_mask];
            }
            compatibleWritableRaster.setDataElements(0, i, width, 1, intArray);
        }
        return new BufferedImage(rgBdefault, compatibleWritableRaster, false, null);
    }
    
    public boolean isValid(int n) {
        return n >= 0 && n < this.map_size && (this.validBits == null || this.validBits.testBit(n));
    }
    
    public boolean isValid() {
        return this.validBits == null;
    }
    
    public BigInteger getValidPixels() {
        if (this.validBits == null) {
            return this.getAllValid();
        }
        return this.validBits;
    }
    
    @Override
    public void finalize() {
    }
    
    @Override
    public String toString() {
        return new String("IndexColorModel: #pixelBits = " + this.pixel_bits + " numComponents = " + this.numComponents + " color space = " + this.colorSpace + " transparency = " + this.transparency + " transIndex   = " + this.transparent_index + " has alpha = " + this.supportsAlpha + " isAlphaPre = " + this.isAlphaPremultiplied);
    }
    
    static {
        IndexColorModel.opaqueBits = new int[] { 8, 8, 8 };
        IndexColorModel.alphaBits = new int[] { 8, 8, 8, 8 };
        ColorModel.loadLibraries();
        initIDs();
    }
}
